@@ -1,6 +1,6 @@
 package Test::SSH;
 
-our $VERSION = '0.06_01';
+our $VERSION = '0.01';
 
 use strict;
 use warnings;
@@ -12,20 +12,9 @@ require Test::More;
 
 my (@extra_path, @default_user_keys, $default_user, $private_dir);
 
-my @default_test_commands = ('true', 'exit', 'echo foo', 'date',
-                             'cmd /c ver', 'cmd /c echo foo');
-
-if ( $^O =~ /^MSWin/) {
+if ( $^O =~ /^Win/) {
     require Win32;
     $default_user = Win32::LoginName();
-	
-	my @pf;
-	for my $folder (qw(PROGRAM_FILES PROGRAM_FILES_COMMON)) {
-		my $dir = eval "Win32::GetFolderPath(Win32::$folder)";
-		if (defined $dir) {
-		    push @extra_path, File::Spec->join($dir, 'PuTTY');
-		}
-	}
 }
 else {
     @extra_path = ( map { File::Spec->join($_, 'bin'), File::Spec->join($_, 'sbin') }
@@ -45,7 +34,6 @@ else {
     $default_user = getpwuid($>);
 
     ($private_dir) = bsd_glob("~/.libtest-ssh-perl", GLOB_TILDE|GLOB_NOCHECK);
-
 }
 
 @default_user_keys = grep {
@@ -69,7 +57,7 @@ my %defaults = ( backends      => [qw(Remote OpenSSH)],
                  port          => 22,
                  host          => 'localhost',
                  user          => $default_user,
-                 test_commands => \@default_test_commands,
+                 test_commands => ['true', 'exit', 'echo foo', 'date'],
                  path          => \@default_path,
                  user_keys     => \@default_user_keys,
                  private_dir   => $private_dir,
@@ -95,7 +83,7 @@ sub new {
         my $class = "Test::SSH::Backend::$be";
         eval "require $class; 1" or die;
         my $sshd = $class->new(%opts) or next;
-        $sshd->_log("connection uri", $sshd->uri(hidden_password => 1));
+        $sshd->_log("connection uri", $sshd->uri(hide_password => 1));
         return $sshd;
     }
     return;
@@ -230,13 +218,6 @@ For instance:
 
   my $sshd = Test::SSH->new(run_server => ($ENV{AUTHOR_TESTING} || $ENV{AUTOMATED_TESTING}));
 
-=item override_server_config => $hash_reference
-
-Key/Value pairs in this hash reference will be used to override the
-defaults for the C<sshd_config> file of the C<sshd> server started. If
-a value in this hash is undef, the respective key of the server config
-will be deleted.
-
 =back
 
 Also, the following environment variables can be used to change the
@@ -298,7 +279,7 @@ The accepted options are as follows:
 
 =over 4
 
-=item hidden_password => 1
+=item hide_password => 1
 
 When this option is set and in case of password authentication, the
 password will be replaced by five asterisks on the returned URI.
@@ -309,10 +290,6 @@ password will be replaced by five asterisks on the returned URI.
 
 Returns the connection parameters as a list of key/value pairs.
 
-=item $sshd-E<gt>server_version
-
-Connects to the server and retrieves its version signature.
-
 =back
 
 =head1 BUGS AND SUPPORT
@@ -347,8 +324,7 @@ Amazon Wish List: L<http://amzn.com/w/1WU1P6IR5QZ42>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2013,2016 by Salvador FandiE<ntilde>o (sfandino@yahoo.com),
-    Andreas KE<ouml>nig (andk@cpan.org)
+Copyright (C) 2013 by Salvador FandiE<ntilde>o (sfandino@yahoo.com)
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.14.2 or,
